"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@graphback/core");
const createResolvers_1 = require("./templates/createResolvers");
const resolverWrapper_1 = require("./templates/resolverWrapper");
const runtimeTemplate_1 = require("./templates/runtimeTemplate");
const writeResolvers_1 = require("./writeResolvers");
const PLUGIN_NAME = 'CRUD_RESOLVER_GENERATOR';
/**
 * Graphback Resolver File Generator Plugin
 *
 * Generates:
 *
 * - default CRUD resolvers for all model types.
 * - blank resolver files to implements custom resolvers
 * - CRUD Services for each type for controling the data sources
 *
 * And then writes to content to resolver files in the server.
 *
 */
class ResolverGeneratorPlugin extends core_1.GraphbackPlugin {
    constructor(pluginConfig) {
        super();
        this.pluginConfig = Object.assign({
            format: 'ts',
            layout: "apollo",
            resolversFileName: 'resolvers',
        }, pluginConfig);
        if (!pluginConfig.outputPath) {
            throw new Error("resolver plugin requires outputPath parameter");
        }
    }
    getPluginName() {
        return PLUGIN_NAME;
    }
    createResources(metadata) {
        const outputResolves = this.generateResolvers(metadata);
        if (outputResolves) {
            writeResolvers_1.writeResolvers(outputResolves, this.pluginConfig);
        }
    }
    generateResolvers(metadata) {
        const models = metadata.getModelDefinitions();
        if (models.length === 0) {
            this.logWarning("Provided schema has no models. Cannot generate resolvers");
            return undefined;
        }
        const generatedResolversFunctions = createResolvers_1.generateCRUDResolversFunctions(models);
        const generatedResolverFile = this.createGeneratedResolversFile(generatedResolversFunctions, this.pluginConfig);
        const contextFile = runtimeTemplate_1.createRuntimeFile(models, this.pluginConfig);
        return {
            resolvers: generatedResolverFile,
            context: contextFile
        };
    }
    createGeneratedResolversFile(resolvers, pluginConfig) {
        const generatedResolvers = resolverWrapper_1.createResolverTemplate(resolvers, pluginConfig);
        return {
            fileName: `resolvers.${pluginConfig.format}`,
            output: generatedResolvers
        };
    }
}
exports.ResolverGeneratorPlugin = ResolverGeneratorPlugin;
//# sourceMappingURL=ResolverGeneratorPlugin.js.map